/* SEMsgConsumer
 * Base class for plugins that feed external systems with
 * app-level notifications.
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 */


#include "SEMsgConsumer.hh"
#include <iostream>
#include "SEMsgProtoDefs.hh"
#include "SimpleDebug.hh"
#include "PluginLoader.hh"
#include <strings.h>
#include <string.h>
#include <stdio.h>





// ------------------------------------
// Utility function, to retrieve the voms auth stuff from a string buffer. Up to 10 fqans are supported
// ------------------------------------
void SEMsgConsumer::unpackAuthstuff(char *inbuf, char *DN, char *voname, char ***fqans, int *nfqans) {
  int i = 0, i1 = 0;
  int l = strlen(inbuf);

  DN[0] = 0;
  voname[0] = 0;
  *fqans = 0;

  // Find the next comma or the end
  while ((i1 < l) && inbuf[i1] != ',') i1++;

  // Get the DN
  memcpy(DN, inbuf+i, i1-i);
  DN[i1-i] = '\0';
  if (i >= l) return;
  i1++;
  i = i1;
  

  // Find the next comma or the end
  while ((i1 < l) && inbuf[i1] != ',') i1++;

  // Get the voname
  memcpy(voname, inbuf+i, i1-i);
  voname[i1-i] = '\0';
  if (i >= l) return;
  i1++;
  i = i1;

  // Loop over the fqans.. allocate an array of 10
  *fqans = (char **)malloc(10*sizeof(char *));
  memset(*fqans, 0, 10*sizeof(char *));
  int j = 0;
  while(1) {
    // Find the next comma or the end
    while ((i1 < l) && inbuf[i1] != ',') i1++;

    // Get the fqan, exit if null string
    if (i1 > i) {
      (*fqans)[j++] = strndup(inbuf+i, i1-i);
      *nfqans = j;
    }
    else return;

    if (i >= l) return;
    i1++;
    i = i1;
  }

  
}
void SEMsgConsumer::freeFqans( char **fqans, int nfqans) {

  for (int i = 0; i < nfqans; i++) {
    if (fqans[i])
      free(fqans[i]);
  }

  free(fqans);
}



// Gets a string field from a message, returns the length of the field or -1 if not found
// If counter < 0 then it does not look for a numbered field
int SEMsgConsumer::getStringField(std::string &val,
				  const cms::MapMessage* m,
				  const char *fieldname,
				  int counter,
				  const char *errordesc) {
  const char *ed = "SEMsgConsumer::getStringField";
  if (errordesc) ed = errordesc;
  
  if (counter < 0) {
    try {

      if (m->itemExists(fieldname)) {
	val = m->getString(fieldname);

	Info(SimpleDebug::kHIGH, ed, fieldname << "=" << val);
	return val.length();
      }
      
    } catch (cms::CMSException& e) {
      Error(ed, "CMSException catched while fetching string" << fieldname << " Exception:" << e.getStackTraceString());
    }


    
    val.clear();
    return -1;
  }

  char s1[1024], s2[1024];
  strcpy(s1, fieldname);
  sprintf(s2, "%.4d", counter);
  strcat(s1, s2);

  try {
    val = m->getString(s1);
  } catch (cms::CMSException& e) {
    Error(ed, "CMSException catched while fetching string" << s1 << " Exception:" << e.getStackTraceString());
    return -1;
  }

  Info(SimpleDebug::kHIGH, ed, s1 << "=" << val);
  return val.length();

}

// Gets an int field from a message, returns > 0 if found, or -1 if not found
// If counter < 0 then it does not look for a numbered field
int SEMsgConsumer::getLongField(long &val,
		const cms::MapMessage* m,
		const char *fieldname,
		int counter,
		const char *errordesc) {

  const char *ed = "SEMsgConsumer::getIntField";
  if (errordesc) ed = errordesc;

   if (counter < 0) {
    try {

      if (m->itemExists(fieldname)) {
	val = (int)m->getInt(fieldname);
	Info(SimpleDebug::kHIGH, ed, fieldname << "=" << val);
	return 1;
      }
      
    } catch (cms::CMSException& e) {
      Error(ed, "CMSException catched while fetching long" << fieldname << " Exception:" << e.getStackTraceString());
    }

    val = 0;
    return -1;
  }


  char s1[1024], s2[1024];
  strcpy(s1, fieldname);
  sprintf(s2, "%.4d", counter);
  strcat(s1, s2);

  try {
    val = (int)m->getInt(s1);
  } catch (cms::CMSException& e) {
    Error(ed, "CMSException catched while fetching long" << s1 << " Exception:" << e.getStackTraceString());
    return -1;
  }

  Info(SimpleDebug::kHIGH, ed, s1 << "=" << val);
  return 1;

}

// Gets a long long field from a message, returns > 0 if found, or -1 if not found
// If counter < 0 then it does not look for a numbered field
int SEMsgConsumer::getLongLongField(long long &val,
		const cms::MapMessage* m,
		const char *fieldname,
		int counter,
		const char *errordesc) {

  const char *ed = "SEMsgConsumer::getLongLongField";
  if (errordesc) ed = errordesc;

   if (counter < 0) {
    try {

      if (m->itemExists(fieldname)) {
	val = m->getLong(fieldname);
	Info(SimpleDebug::kHIGH, ed, fieldname << "=" << val);
	return 1;
      }
      
    } catch (cms::CMSException& e) {
      Error(ed, "CMSException catched while fetching long long" << fieldname << " Exception:" << e.getStackTraceString());
    }

    val = 0;
    return -1;
  }


  char s1[1024], s2[1024];
  strcpy(s1, fieldname);
  sprintf(s2, "%.4d", counter);
  strcat(s1, s2);

  try {
    val = m->getLong(s1);
  } catch (cms::CMSException& e) {
    Error(ed, "CMSException catched while fetching long long" << s1 << " Exception:" << e.getStackTraceString());
    return -1;
  }

  Info(SimpleDebug::kHIGH, ed, s1 << "=" << val);
  return 1;

}



// Override the onMessage method and dispatch to the various callbacks, if defined
void SEMsgConsumer::onMessage( const cms::Message* m ) throw() {

   // The msg found its way, send the acks if needed
   CommitMessages();

   try
   {
     if (DebugLevel() >= SimpleDebug::kHIGH) {
       // Build a dump of the message
       std::vector< std::string > vs = m->getPropertyNames();

       for (unsigned int i = 0; i < vs.size(); i++) {
	 Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage",
	      "Got message. Header field " << i << " " <<
	      vs[i] << ":" <<  m->getStringProperty(vs[i]));
       }
     }

     // See if it's a simple text message, let's print it to the log
     const cms::TextMessage* textMessage =
        dynamic_cast< const cms::TextMessage* >( m );
     if (textMessage != NULL) {
        Info(SimpleDebug::kLOW, "SEMsgConsumer::onMessage", "Got text message: " << textMessage->getText());
        return;
     }

     const cms::MapMessage* mm =
       dynamic_cast< const cms::MapMessage* >( m );
     if (mm == NULL) {
       Error("SEMsgConsumer::onMessage", "Got an invalid message.");
       return;
     }

     Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Got SEMsg message." <<
	  " proto:" <<
	  mm->getIntProperty(PROTO_MAJOR_NAME) << "." <<
	  mm->getIntProperty(PROTO_MINOR_NAME) << "." <<
	  mm->getIntProperty(PROTO_PATCH_NAME) <<
	  " cmd:" << mm->getStringProperty(PROTO_CMD_NAME));

     // Verify if this consumer is allowed to decode the incoming message
     // To do this, we check the proto version, ignoring the patch level
     int msgprotover = 
       mm->getIntProperty(PROTO_MAJOR_NAME)*100 +
       mm->getIntProperty(PROTO_MINOR_NAME)*10;
       
     int myprotover = 
       PROTO_MAJOR*100 +
       PROTO_MINOR*10;

     if (myprotover < msgprotover) {
       Error("SEMsgConsumer::onMessage", "Unsupported protocol version in the message. Skipping.");
       return;
     }

     DumpMessage(mm);

     if (CheckMessage(mm)) {
       Error("SEMsgConsumer::onMessage", "Message signature invalid.");
       return;
     }
       

      const char *cmd = mm->getStringProperty(PROTO_CMD_NAME).c_str();
      
      // Now dispatch the notification to the proper event handler, defined in this class
      if (!strcasecmp(cmd, PROTO_CMD_CHMOD)) {
	onMessageChmod(mm);
	Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
	return;
      }
      if (!strcasecmp(cmd, PROTO_CMD_FILENOTAVAILABLE)) {
	onMessageFileNotAvailable(mm);
	Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
	return;
      }
      if (!strcasecmp(cmd, PROTO_CMD_FILEAVAILABLE)) {
	onMessageFileAvailable(mm);
	Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
	return;
      }
      if (!strcasecmp(cmd, PROTO_CMD_FILEDELETED)) {
        onMessageFileDeleted(mm);
        Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
        return;
      }
      if (!strcasecmp(cmd, PROTO_CMD_ADDREPLICA)) {
	onMessageAddReplica(mm);
	Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
	return;
      }
      if (!strcasecmp(cmd, PROTO_CMD_SETACL)) {
	onMessageSetACL(mm);
	Info(SimpleDebug::kHIGH, "SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " processed.");
	return;
      }
      Error("SEMsgConsumer::onMessage", "Message " <<  mm->getStringProperty(PROTO_CMD_NAME) << " not recognized.");

   } catch (cms::CMSException& e) {
     Error("SEMsgConsumer::onMessage", "CMSException catched. This message will be skipped because it looks malformed. Exception:" << e.getStackTraceString());
   }

}



// Dump all the string fields of the message
void SEMsgConsumer::DumpMessage( const cms::MapMessage* m ) {
  const char *fname="SEMsgConsumer::DumpMessage";


  if (DebugLevel() < SimpleDebug::kHIGHEST) return;

  std::vector< std::string > names = m->getMapNames();

  for (std::vector< std::string >::iterator it = names.begin(); it != names.end(); it++) {
    try {
      Info(SimpleDebug::kHIGHEST, fname, (*it) << "=" << m->getString((*it)) );
    }
    catch (cms::CMSException& e) {
      Error(fname, "CMSException catched while fetching string" << (*it) << " Exception:" << e.getStackTraceString());
    }

  }
}




 



// ------------------------------------------------------------------------------------
// Plugin-related stuff
// ------------------------------------------------------------------------------------

// The plugin functionality. This function invokes the plugin loader, looking for the
// plugin where to call the hook function
SEMsgConsumer *GetSEMsgConsumerPluginClass(char *pluginPath, GetSEMsgConsumerArgs) {
   PluginLoader *myLib;
   SEMsgConsumer *(*ep)(GetSEMsgConsumerArgs);

   // If we have no plugin path then use the default method (this will always
   // succeed).
   if (!pluginPath || !strlen(pluginPath)) {
      Info(SimpleDebug::kMEDIUM, "GetSEMsgConsumerPluginClass", "Getting default plugin.");
      return GetSEMsgConsumer(brokerURI, destname, useTopic, sessionTransacted);
   }


   // Create a plugin object (we will throw this away without deletion because
   // the library must stay open but we never want to reference it again).
   Info(SimpleDebug::kMEDIUM, "GetSEMsgConsumerPluginClass", "Loading plugin " << pluginPath);
   if (!(myLib = new PluginLoader(pluginPath))) {
      Info(SimpleDebug::kLOW, "GetSEMsgConsumerPluginClass", "Failed loading plugin " << pluginPath);
      return NULL;
   }
   
   // Now get the entry point of the object creator
   Info(SimpleDebug::kMEDIUM, "GetSEMsgConsumerPluginClass", "Getting entry point for plugin " << pluginPath);
   ep = (SEMsgConsumer *(*)(GetSEMsgConsumerArgs))(myLib->getPlugin("GetSEMsgConsumer"));
   if (!ep) {
      Info(SimpleDebug::kLOW, "GetSEMsgConsumerPluginClass", "Could not get entry point for plugin " << pluginPath);
      return NULL;
   }

   // Get the Object now
   Info(SimpleDebug::kMEDIUM, "GetSEMsgConsumerPluginClass", "Getting class instance for plugin " << pluginPath);
   SEMsgConsumer *c = ep(brokerURI, destname, useTopic, sessionTransacted);
   if (!c)
      Info(SimpleDebug::kLOW, "GetSEMsgConsumerPluginClass", "Could not get class instance for plugin " << pluginPath);
   return c;

}

extern "C" SEMsgConsumer *GetSEMsgConsumer(GetSEMsgConsumerArgs)
{
   return (SEMsgConsumer *)new SEMsgConsumer(brokerURI, destname, useTopic, sessionTransacted);
}
