/* SEMsgProducer
 * The main API that produces the Catalogue Synchronization messages
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 */

#include "SEMsgProducer.hh"
#include <iostream>
#include "SEMsgProtoDefs.hh"
#include "SimpleDebug.hh"
#include "PluginLoader.hh"
#include <map>
#include <string>



// The useful messages to send
int SEMsgProducer::CanNotifyNotAvailable() {
  return -1; // means true
};
int SEMsgProducer::NotifyNotAvailable(char *authDN, char *filename) {

   localMsg *m = new localMsg((char *)PROTO_CMD_FILENOTAVAILABLE);
   if (!m)  {
      Error("SEMsgProducer::NotifyNotAvailable", "Could not initialize message.");
      return 1;
   }

   // Put in the body the auth info that shall do the operation (generally the user that triggered it)
   if (authDN)
     m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));

   // Now put in the body the information related to this primitive
   m->bodystr.insert(TStrStrPair(PROTO_FILENOTAVAILABLE_FILENAME_S, filename));
 
   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;
};

int SEMsgProducer::CanNotifyAvailable() {
  return -1; // means true
};
int SEMsgProducer::NotifyAvailable(char *authDN, char *filename) {

   localMsg *m = new localMsg((char *)PROTO_CMD_FILEAVAILABLE);
   if (!m)  {
      Error("SEMsgProducer::NotifyNotAvailable", "Could not initialize message.");
      return 1;
   }

   // Put in the body the auth info that shall do the operation (generally the user that triggered it)
   if (authDN)
     m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));

   // Now put in the body the information related to this primitive
   m->bodystr.insert(TStrStrPair(PROTO_FILEAVAILABLE_FILENAME_S, filename));
 
   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;
};

int SEMsgProducer::CanNotifyChmod() {
  return -1; // means true
};
int SEMsgProducer::NotifyChmod(char *authDN, char *filename, int perm) {

   localMsg *m = new localMsg((char *)PROTO_CMD_CHMOD);
   if (!m)  {
      Error("SEMsgProducer::NotifyChmod", "Could not initialize message.");
      return 1;
   }

   // Put in the body the auth info that shall do the operation (generally the user that triggered it)
   if (authDN)
     m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));

   // Now put in the body the information related to this primitive
   m->bodystr.insert(TStrStrPair(PROTO_CHMOD_FILENAME_S, filename));
   m->bodylong.insert(TStrLongPair(PROTO_CHMOD_UNIXPERM_L, perm));

   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;
};

int SEMsgProducer::CanNotifyChown() {
  return -1; // means true
};
int SEMsgProducer::NotifyChown(char *authDN, char *filename, char *username, char *groupname) {

   localMsg *m = new localMsg((char *)PROTO_CMD_CHOWN);
   if (!m)  {
      Error("SEMsgProducer::NotifyChown", "Could not initialize message.");
      return 1;
   }


   // Put in the body the auth info that shall do the operation (generally the user that triggered it)
   if (authDN)
     m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));

   // Now put in the body the information related to this primitive
   m->bodystr.insert(TStrStrPair(PROTO_CHOWN_FILENAME_S, filename));
   m->bodystr.insert(TStrStrPair(PROTO_CHOWN_USERNAME_S, username));
   m->bodystr.insert(TStrStrPair(PROTO_CHOWN_GROUPNAME_S, groupname));

   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;

};

int SEMsgProducer::CanNotifyDeleted() {
  return -1; // means true
};
int SEMsgProducer::NotifyDeleted(char *authDN, char *filename) {

   localMsg *m = new localMsg((char *)PROTO_CMD_FILEDELETED);
   if (!m)  {
      Error("SEMsgProducer::NotifyDeleted", "Could not initialize message.");
      return 1;
   }

   // Put in the body the auth info that shall do the operation (generally the user that triggered it)
   if (authDN)
     m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));


   // Now put in the body the information related to this primitive
   m->bodystr.insert(TStrStrPair(PROTO_CHOWN_FILENAME_S, filename));

   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;

};

int SEMsgProducer::CanNotifyAddReplica() {
  return -1; // means true
};
int SEMsgProducer::NotifyAddReplica(char *authDN, 
				    char      *lfn,
				    char      *guid,
				    int        mode,
				    long long  size,
				    char      *csumtype,
				    char      *csumvalue,
				    char      *server,
				    char      *sfn,
				    char      *status,
				    char      *f_type,
				    char      *poolname,
				    char      *fs) {


    localMsg *m = new localMsg((char *)PROTO_CMD_ADDREPLICA);
    if (!m)  {
      Error("SEMsgProducer::NotifyAddReplica", "Could not initialize message.");
      return 1;
    }

    // Put in the body the auth info that shall do the operation (generally the user that triggered it)
    if (authDN)
      m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));


    // Now put in the body the information related to this primitive
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_LFN_S, lfn));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_GUID_S, guid));
    m->bodylong.insert(TStrLongPair(PROTO_ADDREPLICA_MODE_L, (long)mode ));
    m->bodylonglong.insert(TStrLonglongPair(PROTO_ADDREPLICA_SIZE_LL, size));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_CSUMTYPE_S, csumtype));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_CSUMVALUE_S, csumvalue));

    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_SERVER_S, server));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_SFN_S, sfn));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_STATUS_S, status));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_F_TYPE_S, f_type));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_POOLNAME_S, poolname));
    m->bodystr.insert(TStrStrPair(PROTO_ADDREPLICA_FS_S, fs));

   // And then enqueue the message... yuk!
   sendQ.push_back(m);
   return 0;
}

int SEMsgProducer::CanNotifySetACL() {
  return -1; // means true
};
int SEMsgProducer::NotifySetACL(char *authDN, char *filename, char *acl) {
  localMsg *m = new localMsg((char *)PROTO_CMD_SETACL);
  if (!m)  {
    Error("SEMsgProducer::NotifySetACL", "Could not initialize message.");
    return 1;
  }

  // Put in the body the auth info that shall do the operation (generally the user that triggered it)
  if (authDN)
    m->hdrstr.insert(TStrStrPair(PROTO_AUTHDN, authDN));

  // Now put in the body the information related to this primitive
  m->bodystr.insert(TStrStrPair(PROTO_CHMOD_FILENAME_S, filename));
  m->bodystr.insert(TStrStrPair(PROTO_SETACL_ACLENTRIES_S, acl));
  
  // And then enqueue the message... yuk!
  sendQ.push_back(m);
  return 0;
};














// ------------------------------------------------------------------------------------
// Plugin-related stuff
// ------------------------------------------------------------------------------------

// The plugin functionality. This function invokes the plugin loader, looking for the
// plugin where to call the hook function
SEMsgProducer *GetSEMsgProducerPluginClass(char *pluginPath, GetSEMsgProducerArgs) {
   PluginLoader *myLib;
   SEMsgProducer *(*ep)(GetSEMsgProducerArgs);

   // If we have no plugin path then use the default method (this will always
   // succeed).
   if (!pluginPath || !strlen(pluginPath)) {
      Info(SimpleDebug::kMEDIUM, "GetSEMsgProducerPluginClass", "Getting default plugin.");
      return GetSEMsgProducer(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted);
   }


   // Create a plugin object (we will throw this away without deletion because
   // the library must stay open but we never want to reference it again).
   Info(SimpleDebug::kMEDIUM, "GetSEMsgProducerPluginClass", "Loading plugin " << pluginPath);
   if (!(myLib = new PluginLoader(pluginPath))) {
      Info(SimpleDebug::kLOW, "GetSEMsgProducerPluginClass", "Failed loading plugin " << pluginPath);
      return NULL;
   }
   
   // Now get the entry point of the object creator
   Info(SimpleDebug::kMEDIUM, "GetSEMsgProducerPluginClass", "Getting entry point for plugin " << pluginPath);
   ep = (SEMsgProducer *(*)(GetSEMsgProducerArgs))(myLib->getPlugin("GetSEMsgProducer"));
   if (!ep) {
      Info(SimpleDebug::kLOW, "GetSEMsgProducerPluginClass", "Could not get entry point for plugin " << pluginPath);
      return NULL;
   }

   // Get the Object now
   Info(SimpleDebug::kMEDIUM, "GetSEMsgProducerPluginClass", "Getting class instance for plugin " << pluginPath);
   SEMsgProducer *c = ep(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted);
   if (!c)
      Info(SimpleDebug::kLOW, "GetSEMsgProducerPluginClass", "Could not get class instance for plugin " << pluginPath);
   return c;

}

extern "C" SEMsgProducer *GetSEMsgProducer(GetSEMsgProducerArgs)
{
   activemq::library::ActiveMQCPP::initializeLibrary();
   return (SEMsgProducer *)new SEMsgProducer(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted);
}
