/* TestProducer
 * A simple test for the Message producer
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 *
 *
 * Parameters:
 *    SEMsg_SendACL <URI> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-d <debugLevel>] <file1> <ACL1> ... <fileN> <ACLN>
 *
 *    An example URI is: "tcp://gridmsg001.cern.ch:6166?wireFormat=openwire&transport.commandTracingEnabled=true"
 *
 *
 *
 *
 */


#include "../src/SEMsgProducer.hh"
#include "../src/SimpleDebug.hh"
#include <stdlib.h>
#include <string.h>
#include <vector>

using namespace decaf::lang;

int main (int argc, char *argv[]) {

   if (argc < 5) {
      cout << endl <<
         "Usage: " << endl <<
	"SEMsg_SendACL <URI> <authDN> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-d <debugLevel>] <file1> <ACL1> ... <fileN> <ACLN>" << endl <<
	" where <file> is a filename and <ACL> is an ACL in short form." << endl << endl;
      return 1;
   }

   string name = "SEMsg_SendACL";
   bool useTopic = false;
   int numMsg = 1;
   std::vector<string> filez;
   std::vector<string> aclz;
   string fn = "";
   string acl = "";

   for (int i = 3; i < argc; i++) {

      if (!strcmp(argv[i], "-t")) {
         name = argv[++i];
         useTopic = true;
         continue;
      }
      if (!strcmp(argv[i], "-q")) {
         name = argv[++i];
         useTopic = false;
         continue;
      }
      if (!strcmp(argv[i], "-n")) {
         numMsg = atoi(argv[++i]);
         continue;
      }
      if (!strcmp(argv[i], "-d")) {
         DebugSetLevel(atoi(argv[++i]));
         continue;
      }

      // If we expect a filename, let's take it
      if (fn.empty()) {
	fn = argv[i];
	cout << argv[i] << "->" << fn << endl;
      }
      else {
	// We are expecting an acl
	acl = argv[i];
	cout << fn << " " << acl << endl;
	filez.push_back(fn);
	aclz.push_back(acl);
	fn = "";
	acl = "";
      }


   }


   cout << endl << endl <<
      "Starting." << endl <<
      " URI: " << argv[1] <<
      " authDN: " << argv[2] << endl;
   
   if (useTopic)
      cout << " Topic: ";
   else
      cout << " Queue: ";
   cout << name << endl;

   cout <<
      " numMsg: " << numMsg << endl <<

   cout << endl << endl;

   activemq::library::ActiveMQCPP::initializeLibrary(); 

   time_t starttime = time(0);


   SEMsgProducer *p = (SEMsgProducer *)new SEMsgProducer( SimpleDebug::Instance(), NULL, argv[1], name, useTopic, false);
      if (!p) return 1;


      for (int i = 0; i < numMsg; i++)
	for (unsigned int j = 0; j < filez.size(); j++)
		p->NotifySetACL((char *)argv[2], (char *)filez[j].c_str(), (char *)aclz[j].c_str());


      if (p->Start()) {
         cout << "Connection failed" << endl;
         return 1;
      }

      p->WaitQueueEmpty(0);
      p->Join();
      delete p;


   // Print a small summary
   time_t elapsed = time(0) - starttime;
   cout << endl << endl << "Finished!" << endl;
   cout <<
      "Total sent:           " << numMsg * filez.size() << endl <<
      "Time elapsed:         " << elapsed << "s" << endl;

   cout <<
      "Avg rate:             ";
   if (elapsed > 0)
      cout << numMsg * filez.size() / elapsed << endl;
   else
      cout << "n/a" << endl;

   cout << endl << endl;
   activemq::library::ActiveMQCPP::shutdownLibrary();
   return 0;
}
