/* TestProducer
 * A simple test for the Message producer
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 *
 *
 * Parameters:
 *    SEMsg_SendAddReplica <URI> <filename> <permissions> [-t <topicname>] [-q <queuename>] [-n <numMsg>] <file1> ... <fileN>
 *
 *    An example URI is: "tcp://gridmsg001.cern.ch:6166?wireFormat=openwire&transport.commandTracingEnabled=true"
 *
 *
 *
 *
 */


#include "../src/SEMsgProducer.hh"
#include "../src/SimpleDebug.hh"
#include <stdlib.h>
#include <string.h>
#include <vector>

using namespace decaf::lang;

int main (int argc, char *argv[]) {

   if (argc < 5) {
      cout << endl <<
         "Usage: " << endl <<
	"SEMsg_SendAddReplica <URI> <authDN> [-t <topicname>] [-q <queuename>] [-n <numMsg>] <lfn> <guid> <mode> <size> <csumtype> <csumvalue> <server> <sfn> <status> <f_type> <poolname> <fs>" << endl << endl;
      return 1;
   }

   string name = "SEMsg_SendAddReplica";
   bool useTopic = false;
   int numMsg = 1;

   std::vector<string> lfnz, guidz, csumtypez, csumvaluez, serverz,
     sfnz, statusz, f_typez, poolnamez, fsz;
   std::vector<long long> sizez;
   std::vector<int> modez;

   int parmcnt = 0, item = 0;

   for (int i = 3; i < argc; i++) {

      if (!strcmp(argv[i], "-t")) {
         name = argv[++i];
         useTopic = true;
         continue;
      }
      if (!strcmp(argv[i], "-q")) {
         name = argv[++i];
         useTopic = false;
         continue;
      }
      if (!strcmp(argv[i], "-n")) {
         numMsg = atoi(argv[++i]);
         continue;
      }
      if (!strcmp(argv[i], "-d")) {
         DebugSetLevel(atoi(argv[++i]));
         continue;
      }

      switch (parmcnt) {
      case 0:
	lfnz.push_back( argv[i] );
	parmcnt++;
	break;
      case 1:
	guidz.push_back( argv[i] );
	parmcnt++;
	break;
      case 2:
	modez.push_back( atoi(argv[i]) );
	parmcnt++;
	break;
      case 3:
	sizez.push_back( atoll(argv[i]) );
	parmcnt++;
	break;
      case 4:
	csumtypez.push_back( argv[i] );
	parmcnt++;
	break;
      case 5:
	csumvaluez.push_back( argv[i] );
	parmcnt++;
	break;
      case 6:
	serverz.push_back( argv[i] );
	parmcnt++;
	break;
      case 7:
	sfnz.push_back( argv[i] );
	parmcnt++;
	break;
      case 8:
	statusz.push_back( argv[i] );
	parmcnt++;
	break;
      case 9:
	f_typez.push_back( argv[i] );
	parmcnt++;
	break;
      case 10:
	poolnamez.push_back( argv[i] );
	parmcnt++;
	break;
      case 11:
	fsz.push_back( argv[i] );

	cout <<"Entry: " << i <<
	  lfnz[item] << " " <<
	  guidz[item] << " " <<
	  modez[item] << " " <<
	  csumtypez[item] << " " <<
	  csumvaluez[item] << " " <<
	  serverz[item] << " " <<
	  sfnz[item] << " " <<
	  statusz[item] << " " <<
	  f_typez[item] << " " <<
	  poolnamez[item] << " " <<
	  fsz[item] << endl;

	parmcnt = 0;
	item++;
	break;
      }


   }

   if (parmcnt != 0) {
     cout << endl << "Invalid argument count " << parmcnt << endl << endl;
     exit(1);
   }

   cout << endl << endl <<
      "Starting." << endl <<
      " URI: " << argv[1] << endl;
   
   if (useTopic)
      cout << " Topic: ";
   else
      cout << " Queue: ";
   cout << name << endl;

   cout <<
      " numMsg: " << numMsg << endl <<

   cout << endl << endl;

   activemq::library::ActiveMQCPP::initializeLibrary(); 

   time_t starttime = time(0);


      SEMsgProducer *p = (SEMsgProducer *)new SEMsgProducer( SimpleDebug::Instance(), NULL, argv[1], name, useTopic, false);
      if (!p) return 1;
      if (p->Start()) {
         cout << "Connection failed" << endl;
         return 1;
      }

      for (int i = 0; i < numMsg; i++)
	for (unsigned int j = 0; j < guidz.size(); j++)
		p->NotifyAddReplica(argv[2],
				    (char *)lfnz[j].c_str(),
				    (char *)guidz[j].c_str(),
				    modez[j],
				    sizez[j],
				    (char *)csumtypez[j].c_str(),
				    (char *)csumvaluez[j].c_str(),
				    (char *)serverz[j].c_str(),
				    (char *)sfnz[j].c_str(),
				    (char *)statusz[j].c_str(),
				    (char *)f_typez[j].c_str(),
				    (char *)poolnamez[j].c_str(),
				    (char *)fsz[j].c_str());

      p->WaitQueueEmpty(0);
      p->Join();
      delete p;


   // Print a small summary
   time_t elapsed = time(0) - starttime;
   cout << endl << endl << "Finished!" << endl;
   cout <<
      "Total sent:           " << numMsg * guidz.size() << endl <<
      "Time elapsed:         " << elapsed << "s" << endl;

   cout <<
      "Avg rate:             ";
   if (elapsed > 0)
      cout << numMsg * guidz.size() / elapsed << endl;
   else
      cout << "n/a" << endl;

   cout << endl << endl;
   activemq::library::ActiveMQCPP::shutdownLibrary();
   return 0;
}
