/* TestProducer
 * A simple test for the Message producer
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 *
 *
 * Parameters:
 *    SEMsg_SendDeleted <URI> <filename> [-t <topicname>] [-q <queuename>] [-n <numMsg>] <file1> ... <fileN>
 *
 *    An example URI is: "tcp://dev.msg.cern.ch:6166?wireFormat=openwire&transport.commandTracingEnabled=true"
 *
 *
 *
 *
 */


#include "../src/SEMsgProducer.hh"
#include "../src/SimpleDebug.hh"
#include <stdlib.h>
#include <string.h>
#include <vector>
#include <string>

using namespace decaf::lang;

int main (int argc, char *argv[]) {

   if (argc < 4) {
      cout << endl <<
 	 "Notifies that a set of files (replicas) have been deleted" << endl <<
         " Usage: " << endl <<
         " SEMsg_SendDeleted <URI> <authDN> [-t <topicname>] [-q <queuename>] [-n <numMsg>] <file1> ... <fileN>" <<
         endl << endl;
      return 1;
   }

   string name = "SEMsg_SendDeleted";
   bool useTopic = false;
   int numMsg = 1;
   std::vector<string> filez;


   for (int i = 3; i < argc; i++) {
      if (!strcmp(argv[i], "-t")) {
         name = argv[++i];
         useTopic = true;
         continue;
      }
      if (!strcmp(argv[i], "-q")) {
         name = argv[++i];
         useTopic = false;
         continue;
      }
      if (!strcmp(argv[i], "-n")) {
         numMsg = atoi(argv[++i]);
         continue;
      }
      if (!strcmp(argv[i], "-d")) {
         DebugSetLevel(atoi(argv[++i]));
         continue;
      }
      filez.push_back(argv[i]);

   }


   cout << endl << endl <<
      "Starting." << endl <<
      " URI: " << argv[1] << endl;
   
   if (useTopic)
      cout << " Topic: ";
   else
      cout << " Queue: ";
   cout << name << endl;

   cout <<
      " numMsg: " << numMsg << endl <<

   cout << endl << endl;

   activemq::library::ActiveMQCPP::initializeLibrary(); 

   time_t starttime = time(0);


   SEMsgProducer *p = (SEMsgProducer *)new SEMsgProducer( SimpleDebug::Instance(), NULL, argv[1], name, useTopic, false);
      if (!p) return 1;
      
      if (p->Start()) {
         cout << "Connection failed" << endl;
         return 1;
      }
      
      for (int i = 0; i < numMsg; i++)
	for (unsigned int j = 0; j < filez.size(); j++)
		p->NotifyDeleted(argv[2], (char *)filez[j].c_str());

      p->WaitQueueEmpty(0);
      p->Join();
      delete p;


   // Print a small summary
   time_t elapsed = time(0) - starttime;
   cout << endl << endl << "Finished!" << endl;
   cout <<
      "Total sent:           " << numMsg * filez.size() << endl <<
      "Time elapsed:         " << elapsed << "s" << endl;

   cout <<
      "Avg rate:             ";
   if (elapsed > 0)
      cout << numMsg * filez.size() / elapsed << endl;
   else
      cout << "n/a" << endl;


   cout << endl << endl;
   activemq::library::ActiveMQCPP::shutdownLibrary();
   return 0;
}
