/* TestConsumer
 * A simple test for the Message consumer
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 *
 *
 * Parameters:
 *    TestConsumer <URI> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-mt <maxTestTime>]
 *
 *    An example URI is: "tcp://gridmsg002.cern.ch:6166?wireformat=openwire&transport.commandTracingEnabled=true"
 *
 *
 *
 *
 */


#include "../src/RunnableConsumer.hh"
#include <stdlib.h>
#include <string.h>
#include <vector>

using namespace std;

int main (int argc, char *argv[]) {

   if (argc < 2) {
      cout << endl <<
         "Usage: " << endl <<
         "TestConsumer <URI> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-mt <maxTestTime>]" << endl;
      return 1;
   }

   string name = "SEMsg_TestProducer";
   bool useTopic = false;
   int numMsg = 10; // Exit after having received some msgs
   int maxTestTime = 30000; // 30 secs is the default duration of the test

   for (int i = 2; i < argc; i++) {
      if (!strcmp(argv[i], "-t")) {
         name = argv[++i];
         useTopic = true;
         continue;
      }
      if (!strcmp(argv[i], "-q")) {
         name = argv[++i];
         useTopic = false;
         continue;
      }
      if (!strcmp(argv[i], "-n")) {
         numMsg = atoi(argv[++i]);
         continue;
      }
      if (!strcmp(argv[i], "-mt")) {
         maxTestTime = atoi(argv[++i]);
         continue;
      }
   }


   cout << endl << endl <<
      "Starting." << endl <<
      " URI: " << argv[1] << endl;
   
   if (useTopic)
      cout << " Topic: ";
   else
      cout << " Queue: ";
   cout << name << endl;

   cout << endl << endl;
      
   activemq::library::ActiveMQCPP::initializeLibrary(); 
   
   time_t starttime = time(0);

   RunnableConsumer *c = (RunnableConsumer *)new RunnableConsumer( argv[1], name, useTopic, false);
   if (!c) return 1;

   c->run();
      
      // Print a small summary
   time_t elapsed = time(0) - starttime;
   if (elapsed < 0) elapsed = 0;

   cout << endl << endl << "Finished!" << endl;
   cout <<
      "Total recvd:           " << numMsg << endl <<
      "Time elapsed:          " << elapsed << "s" << endl;

   cout <<
      "Avg rate:              ";
   if (elapsed > 0)
      cout << numMsg / elapsed << endl;
   else
      cout << "n/a" << endl;

   cout << endl << endl;

   activemq::library::ActiveMQCPP::shutdownLibrary();

   return 0;
}
