/* TestProducer
 * A simple test for the Message producer
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 *
 *
 * Parameters:
 *    TestProducer <URI> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-th <numThreads>]
 *
 *    An example URI is: "tcp://gridmsg001.cern.ch:6166?wireformat=openwire&transport.commandTracingEnabled=true"
 *
 *
 *
 *
 */


#include "../src/RunnableProducer.hh"
#include "../src/SimpleDebug.hh"
#include <stdlib.h>
#include <string.h>
#include <vector>

using namespace decaf::lang;

int main (int argc, char *argv[]) {

   if (argc < 3) {
      cout << endl <<
         "Usage: " << endl <<
         "TestProducer <URI> <authDN> [-t <topicname>] [-q <queuename>] [-n <numMsg>] [-th <numThreads>] [-d <loglevel>]" << endl;
      return 1;
   }

   string name = "SEMsg_TestProducer";
   bool useTopic = false;
   int numMsg = 10;
   int numThr = 1;

   for (int i = 3; i < argc; i++) {
      if (!strcmp(argv[i], "-t")) {
         name = argv[++i];
         useTopic = true;
         continue;
      }
      if (!strcmp(argv[i], "-q")) {
         name = argv[++i];
         useTopic = false;
         continue;
      }
      if (!strcmp(argv[i], "-n")) {
         numMsg = atoi(argv[++i]);
         continue;
      }
      if (!strcmp(argv[i], "-d")) {
         DebugSetLevel(atoi(argv[++i]));
         continue;
      }
      if (!strcmp(argv[i], "-th")) {
         numThr = atoi(argv[++i]);
         continue;
      }
   }


   cout << endl << endl <<
      "Starting." << endl <<
      " URI: " << argv[1] << endl;
   
   if (useTopic)
      cout << " Topic: ";
   else
      cout << " Queue: ";
   cout << name << endl;

   cout <<
      " numMsg: " << numMsg << endl <<
      " numThreads: " << numThr << endl;

   cout << endl << endl;

   activemq::library::ActiveMQCPP::initializeLibrary(); 

   time_t starttime = time(0);

   if (numThr == 1) {
      SimpleProducer *p = (SimpleProducer *)new SimpleProducer( argv[1], name, useTopic, false);
      if (!p) return 1;
      
      if (p->Connect()) {
         cout << "Connection failed" << endl;
         return 1;
      }
      
      p->Say((char *)"This is a test message.", numMsg);
      delete p;
   }
   else {

      // The multithreaded case. Create many runnables and store them in a vector
      vector<RunnableProducer *> repo;
      for (int i = 0; i < numThr; i++) {
         RunnableProducer *p = (RunnableProducer *)new RunnableProducer( argv[1], name, useTopic, false);
         if (!p) break;

         p->testSendMsgs = numMsg;

         repo.push_back(p);

         p->Start();
      }


      // Now join the threads as soon as they finish working
      for (unsigned int i = 0; i < repo.size(); i++) {
         cout << "Joining thread " << i << endl;
         repo[i]->Join();
         delete repo[i];
      }
   }

   // Print a small summary
   time_t elapsed = time(0) - starttime;
   cout << endl << endl << "Finished!" << endl;
   cout <<
      "Total sent:           " << numMsg * numThr << endl <<
      "Time elapsed:         " << elapsed << "s" << endl;

   cout <<
      "Avg rate:             ";
   if (elapsed > 0)
      cout << numMsg * numThr / elapsed << endl;
   else
      cout << "n/a" << endl;

   cout <<
      "Avg rate per thread:  ";
   if (elapsed > 0)
      cout << numMsg / elapsed << endl;
   else
      cout << "n/a" << endl;

   cout << endl << endl;
   activemq::library::ActiveMQCPP::shutdownLibrary();
   return 0;
}
